@@ -1,10 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-#
-# Copyright (C) 2011 Loic Jaquemet loic.jaquemet+python@gmail.com
-#
 
-__author__ = "Loic Jaquemet loic.jaquemet+python@gmail.com"
+"""Search for a known structure type in a process memory. """
 
 import argparse
 import logging
@@ -16,10 +13,19 @@ import ctypes
 import subprocess
 import json
 
-import model
+from haystack import model
+from haystack import argparse_utils
 from haystack.memory_mapper import MemoryMapper as MemoryMapper
+from haystack import memory_mapping 
 
-from utils import xrange
+from haystack.utils import xrange
+
+__author__ = "Loic Jaquemet"
+__copyright__ = "Copyright (C) 2012 Loic Jaquemet"
+__email__ = "loic.jaquemet+python@gmail.com"
+__license__ = "GPL"
+__maintainer__ = "Loic Jaquemet"
+__status__ = "Production"
 
 log=logging.getLogger('haystack')
 
@@ -57,11 +63,11 @@ class StructFinder:
     for m in self.targetMappings:
       ##debug, most structures are on head
       log.info("Looking at %s (%d bytes)"%(m, len(m)))
-      if not hasValidPermissions(m):
-        log.warning("Invalid permission for memory %s"%m)
-        continue
-      else:
-        log.debug("%s,%s"%(m,m.permissions))
+      #if not hasValidPermissions(m):
+      #  log.warning("Invalid permission for memory %s. Stil looking at it"%m)
+      #  #continue
+      #else:
+      #  log.debug("%s,%s"%(m,m.permissions))
       log.debug('look for %s'%(structType))
       outputs.extend(self.find_struct_in( m, structType, hintOffset=hintOffset, maxNum=maxNum, maxDepth=maxDepth))
       # check out
@@ -179,6 +185,7 @@ def hasValidPermissions(memmap):
 
 def _callFinder(cmd_line):
   """ Call the haystack finder in a subprocess. Will use pickled objects to communicate results. """
+  log.debug(cmd_line)
   env = os.environ
   env['PYTHONPATH'] = environSep.join(sys.path) # add possible pythonpaths to environnement
   p = subprocess.Popen(cmd_line, stdin=None, stdout=subprocess.PIPE, close_fds=True , env=env)
@@ -252,6 +259,7 @@ def _findStruct(pid=None, memfile=None, memdump=None, structType=None, maxNum=1,
   elif memfile: 
     ### proc mappings dump file
     cmd_line.extend([ "--memfile", memfile] )
+  cmd_line.append('--pickled')
   # always add search
   cmd_line.extend(['search',  '--maxnum', str(int(maxNum))] )
   if fullScan:
@@ -318,6 +326,7 @@ def refreshStruct(pid, structType, offset, debug=False, nommap=False):
   elif memfile: 
     ### proc mappings dump file
     cmd_line.extend([ "--memfile", memfile] )
+  cmd_line.append('--pickled')
   # always add search
   cmd_line.extend(['refresh',  "0x%lx"%offset] )
   instance,validated=_callFinder(cmd_line)
@@ -342,7 +351,7 @@ def argparser():
   target = rootparser.add_mutually_exclusive_group(required=True)
   target.add_argument('--pid', type=int, help='Target PID')
   target.add_argument('--memfile', type=argparse.FileType('r'), help='Use a file memory dump instead of a live process ID')
-  target.add_argument('--dumpfile', type=argparse.FileType('r'), help='Use a haystack memory dump instead of a live process ID')
+  target.add_argument('--dumpname', type=argparse_utils.readable, help='Use a haystack memory dump instead of a live process ID')
 
   output = rootparser.add_mutually_exclusive_group(required=True)
   output.add_argument('--string', dest='human', action='store_const', const=True, help='Print results as human readable string')
@@ -353,7 +362,7 @@ def argparser():
   search_parser = subparsers.add_parser('search', help='search help')
   search_parser.add_argument('--fullscan', action='store_const', const=True, default=False, help='do a full memory scan, otherwise, restrict to the heap')
   search_parser.add_argument('--maxnum', type=int, action='store', default=1, help='Limit to maxnum numbers of results')
-  search_parser.add_argument('--hint', type=int, action='store', default=1, help='hintOffset to start at')
+  search_parser.add_argument('--hint', type=int16, action='store', default=0, help='hintOffset to start at in hex')
   search_parser.set_defaults(func=search)
   #
   refresh_parser = subparsers.add_parser('refresh', help='refresh help')
@@ -362,6 +371,8 @@ def argparser():
   #
   return rootparser
 
+def int16( v):
+  return int(v,16)
 
 def getKlass(name):
   '''
@@ -421,9 +432,18 @@ def search(args):
   if args.fullscan:
     targetMapping = mappings
   else:
-    targetMapping = [m for m in mappings if m.pathname == '[heap]']
+    if args.hint:
+      log.debug('Looking for the mmap containing the hint addr.')
+      m = mappings.getMmapForAddr(args.hint)
+      if not m:
+        log.error('This hint is not a valid addr (0x%x)'%(args.hint))
+        return
+      targetMapping = [m]
+    else:
+      targetMapping = [m for m in mappings if m.pathname == '[heap]']
+    targetMapping = memory_mapping.Mappings(targetMapping, mappings.name)
     if len(targetMapping) == 0:
-      log.warning('No [heap] memorymapping found. Searching everywhere.')
+      log.warning('No memorymapping found. Searching everywhere.')
       targetMapping = mappings
   finder = StructFinder(mappings, targetMapping)
   try:
@@ -535,8 +555,8 @@ def main(argv):
     log.debug("done for pid %d"%opts.pid)
   elif opts.memfile:
     log.debug("done for file %s"%opts.memfile.name)
-  elif opts.dumpfile:
-    log.debug("done for file %s"%opts.dumpfile.name)
+  elif opts.dumpname:
+    log.debug("done for file %s"%opts.dumpname)
   return 0
 
 
