@@ -14,6 +14,7 @@ from PyQt4.Qt import Qt
 from searchinfoStruct import Ui_SearchInfoStructWidget
 import infomodel
 import widgets
+from .. import utils
 
 try:
     _fromUtf8 = QtCore.QString.fromUtf8
@@ -92,6 +93,8 @@ class SearchInfoView(QtGui.QWidget, Ui_SearchInfoStructWidget):
     #QtCore.QObject.connect(self.info_listview, QtCore.SIGNAL("doubleClicked(QModelIndex)"), self.listview_dclicked)    
     QtCore.QObject.connect(self.info_listview, QtCore.SIGNAL("activated(QModelIndex)"), self.listview_activated)    
     #QtCore.QObject.connect(self.info_listview, QtCore.SIGNAL("selectionChanged(QModelSelection)"), self.listview_activated)    
+
+    QtCore.QObject.connect(self.info_tableview, QtCore.SIGNAL("clicked(QModelIndex)"), self.tableview_clicked)    
     pass
   
   def addResult(self, offset, value, color=None):
@@ -135,6 +138,7 @@ class SearchInfoView(QtGui.QWidget, Ui_SearchInfoStructWidget):
         self.info_tableview.setRowHeight(row, 18)
     # enable sorting
     self.info_tableview.setSortingEnabled(True)
+    
     return
    
   def listview_activated (self, qindex):
@@ -146,11 +150,26 @@ class SearchInfoView(QtGui.QWidget, Ui_SearchInfoStructWidget):
     log.info('clicked %s'%(item))
     if self.showDetailledView:
       self._showInfo(self.results[qindex.row()])
+    ### change the hexview
+    # c'est le fun
+    hexedit = self.parent().parent().parent().parent().parent().qhexedit
+    structure = self.results[qindex.row()]
+    hexedit.scrollTo(structure.offset)
+    hexedit.setSelected(structure.offset, len(structure.value) )
     return
 
   def selectionChanged(self, new, old ):
     log.debug('selection changed')
 
+  def tableview_clicked (self, qindex):
+    log.info('clicked %d'%qindex.row())
+    #item = self.info_tableview.model().data(qindex , Qt.DisplayRole)
+    name,value = self.info_tableview.model().arraydata[qindex.row()]
+    ### change the hexview
+    #utils.offsetof(name,)
+    #self.parent().parent().parent().parent().parent().qhexedit.setSelectedscrollTo(val.offset)
+    return
+
 
 
 
