@@ -26,53 +26,60 @@ class Structure(QtGui.QGraphicsItemGroup):
     self.color = color
     log.debug('size: %d'%len(self.value))
     self._makeStruct(self.offset, len(self.value))
-    log.debug('hover: %s handleChildEvents: %s'%( self.acceptsHoverEvents(), self.handlesChildEvents()) )
     self.setAcceptsHoverEvents(True)
     self.setHandlesChildEvents(True)
+    log.debug('hover: %s handleChildEvents: %s'%( self.acceptsHoverEvents(), self.handlesChildEvents()) )
     self.setFlags(QtGui.QGraphicsItem.ItemIsSelectable)
-    self.setToolTip(str(type(value)))
+    if type(value) == str:
+      self.setToolTip(value) # baaaad
+    else:
+      self.setToolTip(str(type(value)))
+
+  def _makeRectItem(self):
+    pen = QtGui.QPen(self.color)
+    brush = QtGui.QBrush(self.color, style=Qt.SolidPattern)
+
+    rectItem = QtGui.QGraphicsRectItem(parent=self) # no need to add to scene
+    rectItem.setAcceptsHoverEvents(True)
+    rectItem.setPen(pen)
+    rectItem.setBrush(brush)
+    return rectItem
   
   def _makeStruct(self, offset, size):
     ''' fix line length to 4K '''
     width = view.LINE_SIZE # use PAGE_SIZE ?
     x1 = (offset % width )
     y = (offset // width )
-    pen = QtGui.QPen(self.color)
-    brush = QtGui.QBrush(self.color, style=Qt.SolidPattern)
+    ############ try to assemble RectF together
+    ##### FIRST LINE
+    # make an item
+    rectItem = self._makeRectItem() # no need to add to scene
     # if offset+xoff > width, draw first line to the end.
     # else, draw the rect and quit
     if (x1 + size > width):
-      #QGraphicsRectItem ( const QRectF & rect, QGraphicsItem * parent = 0 )
-      #self.addToGroup(QtCore.QGraphicsRectItem(QtCore.QRectF(x1, y, width-x1, 1), self.color, QtGui.QBrush(self.color)))
-      '''
-        we should create a QRectF, use it in the QgraphicItem(rect, parent=self)
-        ans use setData on items
-      '''
-      gi = self.scene.addRect(QtCore.QRectF(x1, y, width-x1, 1), pen, brush)
-      # gi is graphics item
-      gi.setAcceptsHoverEvents(True)
-      self.addToGroup(gi)
-      log.debug('line %d : %d,%d,%d,%d first'%(y, x1, y, width-x1, 1))
+      rectItem.setRect( QtCore.QRectF(x1, y, width-x1, 1) )
+      log.debug('line %d : x:%d,y:%d,w:%d,h:%d first'%(y, x1, y, width-x1, 1))
     else:
-      gi = self.scene.addRect(QtCore.QRectF(x1, y, size, 1), pen, brush)
-      gi.setAcceptsHoverEvents(True)
-      self.addToGroup(gi)
-      #log.debug('line 1 : %d,%d,%d,%d stop'%(x1, y, size, 1))
+      rectItem.setRect( QtCore.QRectF(x1, y, size, 1) )
+      log.debug('line %d : x:%d,y:%d,w:%d,h:%d first'%(y, x1, y, width-x1, 1))
       return
+    ##### MIDDLE LINES
     # then draw big rect full lines from ya = y+1  to yb = ((offset+size) // width) - 1
-    yf = ((offset+size) // width)
+    yf = ((offset+size) // width) # on calcule la ligne de fin
+    log.debug('yf:%d offset:%d size:%d width:%d y:%d'%(    yf,offset,size,width, y) )
     if ( yf > y+1 ):
       for ya in xrange(y+1, yf):
-        gi = self.scene.addRect(QtCore.QRectF(0, ya, width, 1), pen, brush)
-        gi.setAcceptsHoverEvents(True)
-        self.addToGroup(gi)
-        log.debug('line %d : %d,%d,%d,%d'%(ya, 0, ya, width, 1))
+        # make an item
+        rectItem = self._makeRectItem() # no need to add to scene      
+        rectItem.setRect( QtCore.QRectF(0, ya, width, 1) )
+        log.debug('line %d : x:%d,y:%d,w:%d,h:%d first'%(ya-y, 0, ya, width, 1))
+    ##### LAST LINE
     # then draw last line from x = 0 to x = offset+size // width
     xf = ((offset+size) % width )
-    gi = self.scene.addRect(QtCore.QRectF(0, yf, width-xf, 1), pen, brush)
-    gi.setAcceptsHoverEvents(True)
-    self.addToGroup(gi)
-    log.debug('line %d : %d,%d,%d,%d stop'%(yf, 0, yf, width-xf, 1))
+    # make an item
+    rectItem = self._makeRectItem() # no need to add to scene      
+    rectItem.setRect( QtCore.QRectF(0, yf, xf, 1) )
+    log.debug('line %d : x:%d,y:%d,w:%d,h:%d first'%(yf, 0, yf, xf, 1))
     return
 
   def onSelect(self):
