@@ -1,4 +1,31 @@
+#!/usr/bin/env python
+# -*- coding: utf-8 -*-
 
+'''Provide several memory mapping wrappers to handle different situations.
+
+Short story, the memory of a process is segmented in several memory 
+zones called memory mapping, 
+  exemple: the heap, the stack, mmap(2)-s of files, mmap(2)-ing a 
+           dynamic library, etc.
+Theses memory mapping represent the memory space of a process. Each 
+mapping hasca start and a end address, which gives boundaries for the 
+range of valid pointer values.
+
+There are several ways to wraps around a memory mapping, given the precise 
+scenario you are in. You could need a wrapper for a live process debugging, a
+wrapper for a mapping that has been dumps in a file, a wrapper for a mapping 
+that has been remapped to memory, etc.
+
+Classes:
+- MemoryMapping : memory mapping metadata
+- ProcessMemoryMapping: memory space from a live process with the possibility to mmap the memspace at any moment.
+- LocalMemoryMapping .fromAddress: memorymapping that lives in local space in a ctypes buffer. 
+- MemoryDumpMemoryMapping .fromFile : memory space from a raw file, with lazy loading capabilities.
+- FileBackedMemoryMapping .fromFile : memory space based on a file, with direct read no cache from file.
+
+This code first 150 lines is mostly inspired by python ptrace by Haypo / Victor Skinner.
+Its intended to be retrofittable with ptrace's memory mappings.
+'''
 
 import os
 import logging
@@ -8,22 +35,20 @@ import struct
 import mmap
 from weakref import ref
 
-# local
-from dbg import openProc, ProcError, ProcessError, HAS_PROC, formatAddress 
-import utils
+# haystack
+from haystack.dbg import openProc, ProcError, ProcessError, HAS_PROC, formatAddress 
+from haystack import utils
+
+__author__ = "Loic Jaquemet"
+__copyright__ = "Copyright (C) 2012 Loic Jaquemet"
+__email__ = "loic.jaquemet+python@gmail.com"
+__license__ = "GPL"
+__maintainer__ = "Loic Jaquemet"
+__status__ = "Production"
+__credits__ = ["Victor Skinner"]
 
 log = logging.getLogger('memory_mapping')
 
-'''
-Memory mappings.
-- MemoryMapping : memory mapping metadata
-- ProcessMemoryMapping: memory space from a live process with the possibility to mmap the memspace at any moment.
-- LocalMemoryMapping .fromAddress: memorymapping that lives in local space in a ctypes buffer. 
-- MemoryDumpMemoryMapping .fromFile : memory space from a raw file, with lazy loading capabilities.
-- FileBackedMemoryMapping .fromFile : memory space based on a file, with direct read no cache from file.
-'''
-
-
 PROC_MAP_REGEX = re.compile(
     # Address range: '08048000-080b0000 '
     r'([0-9a-f]+)-([0-9a-f]+) '
@@ -82,10 +107,8 @@ class MemoryMapping:
       return self.start <= address < self.end
 
   def __str__(self):
-    text = "%s-%s" % (formatAddress(self.start), formatAddress(self.end))
-    if self.pathname:
-      text += " => %s" % self.pathname
-    text += " (%s)" % self.permissions
+    text = ' '.join([formatAddress(self.start), formatAddress(self.end), self.permissions,
+           '0x%0.8x'%(self.offset), '%0.2x:%0.2x'%(self.major_device, self.minor_device), '%0.7d'%(self.inode), str(self.pathname)])
     return text
 
   __repr__ = __str__
@@ -141,8 +164,11 @@ class MemoryMapping:
     return ''.join(string), truncated
 
   def vtop(self, vaddr):
-    return vaddr - self.start
-    
+    ret = vaddr - self.start
+    if ret<0 or ret>len(self):
+      raise ValueError('%x/%x is not a valid vaddr for me'%(vaddr,ret))
+    return ret
+  
   # ---- to implement if needed
   def readWord(self, address):
     raise NotImplementedError
@@ -153,7 +179,6 @@ class MemoryMapping:
   def readArray(self, address, basetype, count):
     raise NotImplementedError
 
-
 class ProcessMemoryMapping(MemoryMapping):
   """
   Process memory mapping (metadata about the mapping).
@@ -201,8 +226,8 @@ class ProcessMemoryMapping(MemoryMapping):
   def mmap(self):
     ''' mmap-ed access gives a 20% perf increase on by tests '''
     if not self.isMmaped():
+      self._process().readArray(self.start, ctypes.c_ubyte, len(self) ) # keep ref
       self._local_mmap_content = self._process().readArray(self.start, ctypes.c_ubyte, len(self) ) # keep ref
-      #self._local_mmap = self._process().read(self.start, self.end-self.start)
       self._local_mmap = LocalMemoryMapping.fromAddress( self, ctypes.addressof(self._local_mmap_content) )
       self._base = self._local_mmap
     return self._local_mmap
@@ -227,13 +252,16 @@ class LocalMemoryMapping(MemoryMapping):
   """
   def __init__(self, address, start, end, permissions, offset, major_device, minor_device, inode, pathname):
     MemoryMapping.__init__(self, start, end, permissions, offset, major_device, minor_device, inode, pathname)
-    self._local_mmap = (ctypes.c_byte * len(self)).from_address(address)
+    self._local_mmap = (ctypes.c_byte * len(self)).from_address(address) # DEBUG TODO byte or ubyte 
     self._address = ctypes.addressof(self._local_mmap)
     #self._vbase = self.start + self._address # shit, thats wraps up...
     self._bytebuffer = None
 
   def vtop(self, vaddr):
-    return vaddr - self.start + self._address 
+    ret = vaddr - self.start + self._address 
+    if ret<self._address or ret>(self._address+len(self)):
+      raise ValueError('%x/%x is not a valid vaddr for me'%(vaddr,ret))
+    return ret
 
   def mmap(self):
     return self
@@ -244,11 +272,16 @@ class LocalMemoryMapping(MemoryMapping):
     word = MemoryMapping.WORDTYPE.from_address(laddr).value # is non-aligned a pb ?, indianess is at risk
     return word
 
-  def readBytes(self, vaddr, size):
-    laddr = vaddr - self.start
-    #data = b''.join([ struct.pack('B',x) for x in self._local_mmap[laddr:laddr+size] ])
-    data = b''.join([ struct.pack('B',x) for x in self.readArray( vaddr, ctypes.c_ubyte, size) ] )
+  def readBytes1(self, vaddr, size):
+    laddr = self.vtop( vaddr )
+    #data = b''.join([ struct.pack('B',x) for x in self.readArray( vaddr, ctypes.c_ubyte, size) ] )
+    data = ctypes.string_at(laddr, size) # real 0.5 % perf
     return data
+
+  def readBufferBytes(self, vaddr, size):
+    laddr = vaddr - self.start
+    return self._bytebuffer[laddr:laddr+size]
+  readBytes = readBytes1
   
   def readStruct(self, vaddr, struct):
     laddr = self.vtop( vaddr )
@@ -263,6 +296,7 @@ class LocalMemoryMapping(MemoryMapping):
   def getByteBuffer(self):
     if self._bytebuffer is None:
       self._bytebuffer = self.readBytes( self.start , len(self))
+      self.readBytes = self.readBufferBytes
     return self._bytebuffer
 
   def initByteBuffer(self, data=None):
@@ -301,13 +335,20 @@ class MemoryDumpMemoryMapping(MemoryMapping):
   def __init__(self, memdump, start, end, permissions='rwx-', offset=0x0, major_device=0x0, minor_device=0x0, inode=0x0, pathname='MEMORYDUMP', preload=False):
     MemoryMapping.__init__(self, start, end, permissions, offset, major_device, minor_device, inode, pathname)
     self._memdump = memdump
+    log.debug('memdump %s'%( memdump))
     self._base = None
-    s = len(LazyMmap(self._memdump))
-    if offset > s:
-      raise ValueError('offset 0x%x too big for filesize 0x%x'%(offset, s))
     if preload:
       self._mmap()
   
+  def useByteBuffer(self):
+    # toddo use bitstring
+    self._mmap().getByteBuffer() ## XXX FIXME buggy
+    # force readBytes update
+    self.readBytes = self._base.readBytes
+  
+  def getByteBuffer(self):
+    return self._mmap().getByteBuffer() 
+  
   def isMmaped(self):
     return not (self._base is None)
     
@@ -325,18 +366,26 @@ class MemoryDumpMemoryMapping(MemoryMapping):
     # mmap.mmap has a full bytebuffer API, so we can use it as is for bytebuffer.
     # we have to get a ctypes pointer-able instance to make our ctypes structure read efficient.
     # sad we can't have a bytebuffer from that same raw memspace
-    # we do not keep the btyebuffer in memory, because it's a lost of space in most cases.
+    # we do not keep the bytebuffer in memory, because it's a lost of space in most cases.
     if self._base is None:
-      if hasattr(self._memdump,'fileno'): # normal file. mmap kinda useless i suppose.
-        log.warning('Memory Mapping content mmap-ed() (double copy) : %s'%(self))
+      mmap_hack = True
+      if mmap_hack: # XXX that is the most fucked up, non-portable fuck I ever wrote.
+        self._local_mmap_bytebuffer = mmap.mmap(self._memdump.fileno(), self.end-self.start, access=mmap.ACCESS_READ)
+        # yeap, that right, I'm stealing the pointer value. DEAL WITH IT.
+        heapmap = struct.unpack('L', (ctypes.c_uint).from_address(id(self._local_mmap_bytebuffer) + 8 ) )[0] 
+        self._local_mmap_content = (ctypes.c_ubyte*(self.end-self.start)).from_address(heapmap)
+      elif hasattr(self._memdump,'fileno'): # normal file. mmap kinda useless i suppose.
+        log.warning('Memory Mapping content mmap-ed() (double copy of %s) : %s'%(self._memdump.__class__, self))
+        # we have the bytes
         local_mmap_bytebuffer = mmap.mmap(self._memdump.fileno(), self.end-self.start, access=mmap.ACCESS_READ)
+        # we need an ctypes
         self._local_mmap_content = utils.bytes2array(local_mmap_bytebuffer, ctypes.c_ubyte)
       else: # dumpfile, file inside targz ... any read() API really
         self._local_mmap_content = utils.bytes2array(self._memdump.read(), ctypes.c_ubyte)
         log.warning('Memory Mapping content copied to ctypes array : %s'%(self))
       # make that _base
       self._base = LocalMemoryMapping.fromAddress( self, ctypes.addressof(self._local_mmap_content) )
-      log.info('LocalMemoryMapping done.')
+      log.debug('LocalMemoryMapping done.')
     #redirect stuff
     self.readWord = self._base.readWord
     self.readArray = self._base.readArray
@@ -356,6 +405,24 @@ class MemoryDumpMemoryMapping(MemoryMapping):
   def readArray(self, vaddr, basetype, count):
     return self._mmap().readArray(vaddr, basetype, count)
 
+  def __getstate__(self):
+    d = dict(self.__dict__)
+    if hasattr(self,'_memdump.name'):
+      d['_memdump_filename'] = self._memdump.name
+    d['_memdump'] = None
+    d['_local_mmap'] = None
+    d['_local_mmap_content'] = None
+    d['_base'] = None
+    d['_process'] = None
+    return d
+  
+  #def __setstate__(self, dic):
+  #  print 'set', dic
+  #  self._memdump = file(dic['_memdump_filename'],'r')
+  #  s = len(LazyMmap(self._memdump))
+  #  return
+
+  
   @classmethod
   def fromFile(cls, memoryMapping, aFile):
     '''
@@ -373,26 +440,32 @@ class FileBackedMemoryMapping(MemoryDumpMemoryMapping):
   def __init__(self, memdump, start, end, permissions='rwx-', offset=0x0, major_device=0x0, minor_device=0x0, inode=0x0, pathname='MEMORYDUMP'):
     MemoryDumpMemoryMapping.__init__(self, memdump, start, end, permissions, offset, major_device, minor_device, inode, pathname, preload=False)
     self._local_mmap = LazyMmap(self._memdump)
+    log.debug( 'FileBackedMemoryMapping created')
     return
 
   def _mmap(self):
     ''' returns self to force super() to read through us  '''
     return self
 
+
   def vtop(self, vaddr):
-    return vaddr - self.start
+    ret = vaddr - self.start
+    if ret<0 or ret>len(self):
+      raise ValueError('%x/%x is not a valid vaddr for me'%(vaddr,ret))
+    return ret
 
   def readBytes(self, vaddr, size):
     laddr = self.vtop(vaddr)
-    size = ctypes.sizeof((ctypes.c_ubyte *count))
+    size = ctypes.sizeof((ctypes.c_ubyte *size))
     data = b''.join([ struct.pack('B',x) for x in self._local_mmap[laddr:laddr+size] ])
     return data
 
   def readStruct(self, vaddr, structType):
     laddr = self.vtop(vaddr)
     size = ctypes.sizeof(structType)
-    stspace = structType.from_buffer_copy(self._local_mmap[laddr:laddr+size], 0).value # is non-aligned a pb ?
-    return self._mmap().readStruct(vaddr, structType)
+    ###WTF is that ? stspace = structType.from_buffer_copy(self._local_mmap[laddr:laddr+size], 0).value # is non-aligned a pb ?
+    ###WTF return self._mmap().readStruct(vaddr, structType)
+    return structType.from_buffer_copy(self._local_mmap[laddr:laddr+size], 0)
 
   def readWord(self, vaddr):
     """Address have to be aligned!"""
@@ -405,7 +478,7 @@ class FileBackedMemoryMapping(MemoryDumpMemoryMapping):
     size = ctypes.sizeof((basetype *count))
     array = (basetype *count).from_buffer_copy(self._local_mmap[laddr:laddr+size], 0)
     return array
-
+  
   @classmethod
   def fromFile(self, memoryMapping, memdump):
     """
@@ -430,8 +503,9 @@ class LazyMmap:
     except OverflowError:
       memdump.seek(os.fstat(memdump.fileno()).st_size)
     self.size = memdump.tell()
-    self.memdump = memdump
+    self.memdump_name = memdump.name
     memdump.seek(i)
+    memdump.close()
   
   def __len__(self):
     return self.size
@@ -448,13 +522,58 @@ class LazyMmap:
     return self._get(start, size)
   
   def _get(self, offset,size):
-    self.memdump.seek(offset)
-    #me = mmap.mmap(memdump.fileno(), end-start, access=mmap.ACCESS_READ)
-    me = utils.bytes2array(self.memdump.read(size) ,ctypes.c_ubyte)
+    memdump = file(self.memdump_name, 'rb')
+    memdump.seek(offset)
+    me = utils.bytes2array(memdump.read(size) ,ctypes.c_ubyte)
+    memdump.close()
     return me
 
 
 
+class Mappings:
+  def __init__(self, lst, filename):
+    if type(lst) != list:
+      raise TypeError('Please feed me a list')
+    self.mappings = lst
+    self.name = filename
+
+  def getMmap(self, pathname):
+    mmap = None
+    if len(self.mappings) >= 1:
+      mmap = [m for m in self.mappings if m.pathname == pathname]
+    if len(mmap) < 1:
+      raise IndexError('No mmap of pathname %s'%(pathname))
+    return mmap
+
+  def getMmapForAddr(self, vaddr):
+    for m in self.mappings:
+      if vaddr in m:
+        return m
+    return False
+
+  def getHeap(self):
+    heap = self.getMmap('[heap]')[0]
+    return heap
+  def getStack(self):
+    stack = self.getMmap('[stack]')[0] 
+    return stack
+
+  def __contains__(self, vaddr):
+    for m in self.mappings:
+      if vaddr in m:
+        return True
+    return False
+
+  def __len__(self):
+    return len(self.mappings)
+  def __getitem__(self, i):
+    return self.mappings[i]
+  def __setitem__(self,i,val):
+    raise NotImplementedError()
+  def __iter__(self):
+    return iter(self.mappings)
+  
+
 def readProcessMappings(process):
   """
   Read all memory mappings of the specified process.
@@ -477,7 +596,7 @@ def readProcessMappings(process):
       line = line.rstrip()
       match = PROC_MAP_REGEX.match(line)
       if not match:
-        raise ProcessError(process, "Unable to parse memoy mapping: %r" % line)
+        raise ProcessError(process, "Unable to parse memory mapping: %r" % line)
       map = ProcessMemoryMapping(
         process,
         int(match.group(1), 16),
@@ -492,5 +611,5 @@ def readProcessMappings(process):
   finally:
     if type(mapsfile) is file:
       mapsfile.close()
-  return maps
+  return Mappings(maps, maps[0].pathname)
 
