@@ -74,7 +74,6 @@ Example :
       "iqmp": [NotNull]
     }
     def loadMembers(self, mappings, maxDepth):
-      print 'example'
       if not LoadableMembers.loadMembers(self, mappings, maxDepth):
         log.debug('RSA not loaded')
         return False
@@ -124,11 +123,16 @@ Enjoy.
 
 '''
 
-__author__ = "Loic Jaquemet loic.jaquemet+python@gmail.com"
-
 import ctypes
 import logging
-from utils import *
+from haystack.utils import *
+
+__author__ = "Loic Jaquemet"
+__copyright__ = "Copyright (C) 2012 Loic Jaquemet"
+__email__ = "loic.jaquemet+python@gmail.com"
+__license__ = "GPL"
+__maintainer__ = "Loic Jaquemet"
+__status__ = "Production"
 
 log = logging.getLogger('model')
 
@@ -192,7 +196,7 @@ def keepRef(obj,typ=None,origAddr=None):
     else:
       origAddr=hex(origAddr)
     if typ is not None:
-      log.warning('references already in cache %s/%s'%(typ,origAddr))
+      log.debug('references already in cache %s/%s'%(typ,origAddr))
     return
   __book.addRef(obj,typ,origAddr)
   return
@@ -242,6 +246,29 @@ class LoadableMembers(ctypes.Structure):
   '''
   classRef=dict()
   expectedValues=dict()
+  
+  ''' 
+    Iterate over the fields and types of this structure, including inherited ones.
+  '''
+  def getFields(self):
+    return type(self).getFields()
+  
+  @classmethod
+  def getFields(cls):
+    mro = cls.mro()[:-3] # cut Structure, _CData and object
+    mro.reverse()
+    me = mro.pop(-1)
+    for typ in mro: # firsts are first, cls is in here in [-1]
+      if not hasattr(typ, '_fields_'):
+        continue
+      for name,vtyp in typ.getFields():
+        #yield ('%s_%s'%(typ.__name__, name), vtyp)
+        yield (name, vtyp)
+    # print mines.
+    for f in me._fields_:
+      yield (f[0],f[1])
+    
+    raise StopIteration
 
   def isValid(self,mappings):
     ''' 
@@ -272,11 +299,12 @@ class LoadableMembers(ctypes.Structure):
     return valid
 
   def _isValid(self,mappings):
-    ''' real implementation.  '''
-    # precheck for quick unvalidation
-    # myfields=dict(self._fields_)  ## some fields are initialised...
-    _fieldsTuple = [ (f[0],f[1]) for f in self._fields_] 
+    ''' real implementation.  check expectedValues first, then the other fields '''
+    log.debug(' ------ isValid ----------')
+    _fieldsTuple = self.getFields()
     myfields=dict(_fieldsTuple)
+    done=[]
+    # check expectedValues first
     for attrname, expected in self.expectedValues.iteritems():
       attrtype = myfields[attrname]
       attr=getattr(self,attrname)
@@ -284,47 +312,48 @@ class LoadableMembers(ctypes.Structure):
         continue
       if not self._isValidAttr(attr,attrname,attrtype,mappings):
         return False
-    #if len(self.expectedValues) >0 :
-    #  log.info('maybe valid . full validation follows validated :%s'%(self.expectedValues.keys()))    
-    # normal check
-    for attrname,attrtype, in _fieldsTuple:
+      done.append(attrname)
+    # check the rest for validation
+    todo = [ (name, typ) for name,typ in self.getFields() if name not in done ]
+    for attrname,attrtype, in todo:
       attr=getattr(self,attrname)
-      # get expected values
       if attrname in self.expectedValues:
-        # shortcut
         if self.expectedValues[attrname] is IgnoreMember:
-          continue # oho ho
-      # validate
+          continue
       if not self._isValidAttr(attr,attrname,attrtype,mappings):
         return False
-      #continue
-    # loop done
+    # validation done
     return True
     
   def _isValidAttr(self,attr,attrname,attrtype,mappings):
     ''' Validation of a single member '''
     # a) 
-    if isBasicType(attr):
+    log.debug('valid: %s'%(attrtype))
+    if isBasicType(attrtype):
       if attrname in self.expectedValues:
         if attr not in self.expectedValues[attrname]:
-          log.debug('%s %s %s bad value not in self.expectedValues[attrname]:'%(attrname,attrtype,repr(attr) ))
+          log.debug('basicType: %s %s %s bad value not in self.expectedValues[attrname]:'%(attrname,attrtype,repr(attr) ))
           return False
-      log.debug('%s %s %s ok'%(attrname,attrtype,repr(attr) ))
+      log.debug('basicType: %s %s %s ok'%(attrname,attrtype,repr(attr) ))
       return True
     # b)
-    if isStructType(attr):
+    if isStructType(attrtype):
       ### do i need to load it first ? becaus it should be memcopied with the super()..
       if not attr.isValid(mappings):
-        log.debug('%s %s %s isValid FALSE'%(attrname,attrtype,repr(attr) ))
+        log.debug('structType: %s %s %s isValid FALSE'%(attrname,attrtype,repr(attr) ))
         return False
-      log.debug('%s %s %s isValid TRUE'%(attrname,attrtype,repr(attr) ))
+      log.debug('structType: %s %s %s isValid TRUE'%(attrname,attrtype,repr(attr) ))
       return True
     # c)
-    if isBasicTypeArrayType(attr):
-      #log.info('%s is arraytype %s we decided it was valid',attrname,repr(attr))#
+    if isBasicTypeArray(attr):
+      if attrname in self.expectedValues:
+        if attr not in self.expectedValues[attrname]:
+          log.debug('basicArray: %s %s %s bad value not in self.expectedValues[attrname]:'%(attrname,attrtype,repr(attr) ))
+          return False
+      log.debug('basicArray: %s is arraytype %s we decided it was valid',attrname,repr(attr))#
       return True
-    if isArrayType(attr):
-      log.debug('%s is arraytype %s recurse validate'%(attrname,repr(attr)) )#
+    if isArrayType(attrtype):
+      log.debug('array: %s is arraytype %s recurse validate'%(attrname,repr(attr)) )#
       attrLen=len(attr)
       if attrLen == 0:
         return True
@@ -334,36 +363,42 @@ class LoadableMembers(ctypes.Structure):
           return False
       return True
     # d)
-    if isCStringPointer(attr):
+    if isCStringPointer(attrtype):
       myaddress=getaddress(attr.ptr)
       if attrname in self.expectedValues:
         # test if NULL is an option
         if not bool(myaddress) :
           if not ( (None in self.expectedValues[attrname]) or
                    (0 in self.expectedValues[attrname]) ):
-            log.debug('%s %s %s isNULL and that is NOT EXPECTED'%(attrname,attrtype,repr(attr) ))
+            log.debug('str: %s %s %s isNULL and that is NOT EXPECTED'%(attrname,attrtype,repr(attr) ))
             return False
-          log.debug('%s %s %s isNULL and that is OK'%(attrname,attrtype,repr(attr) ))
+          log.debug('str: %s %s %s isNULL and that is OK'%(attrname,attrtype,repr(attr) ))
           return True
       if (myaddress != 0) and ( not is_valid_address_value( myaddress, mappings) )   :
-        log.debug('%s %s %s 0x%lx INVALID'%(attrname,attrtype, repr(attr) ,myaddress))
+        log.debug('str: %s %s %s 0x%lx INVALID'%(attrname,attrtype, repr(attr) ,myaddress))
         return False
-      log.debug('%s %s %s is at 0x%lx OK'%(attrname,attrtype,repr(attr),myaddress ))
+      log.debug('str: %s %s %s is at 0x%lx OK'%(attrname,attrtype,repr(attr),myaddress ))
       return True
     # e) 
-    if isPointerType(attr):
+    if isPointerType(attrtype):
       if attrname in self.expectedValues:
         # test if NULL is an option
+        log.debug('isPointerType: bool(attr):%s attr:%s'%(bool(attr), attr))
         if not bool(attr):
           if not ( (None in self.expectedValues[attrname]) or
                    (0 in self.expectedValues[attrname]) ):
-            log.debug('%s %s %s isNULL and that is NOT EXPECTED'%(attrname,attrtype,repr(attr) ))
+            log.debug('ptr: %s %s %s isNULL and that is NOT EXPECTED'%(attrname,attrtype,repr(attr) ))
             return False
-          log.debug('%s %s %s isNULL and that is OK'%(attrname,attrtype,repr(attr) ))
+          log.debug('ptr: %s %s %s isNULL and that is OK'%(attrname,attrtype,repr(attr) ))
           return True
       # all case, 
       _attrType=None
-      if attrtype not in self.classRef:
+      if isVoidPointerType(attrtype):
+        log.debug('Its a simple type. Checking mappings only.')
+        if getaddress(attr) != 0 and not is_valid_address_value( attr, mappings): # NULL can be accepted
+          log.debug('voidptr: %s %s %s 0x%lx INVALID simple pointer'%(attrname,attrtype, repr(attr) ,getaddress(attr)))
+          return False
+      elif attrtype not in self.classRef:
         log.debug("I can't know the size of the basic type behind the %s pointer, it's not a pointer to known registered struct type"%(attrname))
         _attrType=None
       else:
@@ -371,13 +406,13 @@ class LoadableMembers(ctypes.Structure):
         _attrType=self.classRef[attrtype]
       #log.debug(" ihave decided on pointed attrType to be %s"%(_attrType))
       if ( not is_valid_address( attr, mappings, _attrType) ) and (getaddress(attr) != 0):
-        log.debug('%s %s %s 0x%lx INVALID'%(attrname,attrtype, repr(attr) ,getaddress(attr)))
+        log.debug('ptr: %s %s %s 0x%lx INVALID'%(attrname,attrtype, repr(attr) ,getaddress(attr)))
         return False
       # null is accepted by default 
-      log.debug('%s %s 0x%lx OK'%(attrname,repr(attr) ,getaddress(attr)))
+      log.debug('ptr: %s %s 0x%lx OK'%(attrname,repr(attr) ,getaddress(attr)))
       return True
     # ?
-    if isUnionType(attr):
+    if isUnionType(attrtype):
       #log.warning('Union are not validated , yet ')
       return True
     log.error('What type are You ?: %s/%s'%(attrname,attrtype))
@@ -389,9 +424,9 @@ class LoadableMembers(ctypes.Structure):
       A c_void_p cannot be load generically, You have to take care of that.
     '''
     attrtype=type(attr)
-    return ( (isPointerType(attr) and ( attrtype in self.classRef) and bool(attr) and not isFunctionType(attr) ) or
-              isStructType(attr)  or isCStringPointer(attr) or
-              (isArrayType(attr) and not isBasicTypeArrayType(attr) ) ) # should we iterate on Basictypes ? no
+    return ( (isPointerType(attrtype) and ( attrtype in self.classRef) and bool(attr) and not isFunctionType(attrtype) ) or
+              isStructType(attrtype)  or isCStringPointer(attrtype) or
+              (isArrayType(attrtype) and not isBasicTypeArray(attr) ) ) # should we iterate on Basictypes ? no
 
   def loadMembers(self, mappings, maxDepth):
     ''' 
@@ -413,8 +448,7 @@ class LoadableMembers(ctypes.Structure):
       return False
     log.debug('%s do loadMembers ----------------'%(self.__class__.__name__))
     ## go through all members. if they are pointers AND not null AND in valid memorymapping AND a struct type, load them as struct pointers
-    _fieldsTuple = [ (f[0],f[1]) for f in self._fields_] 
-    for attrname,attrtype in _fieldsTuple:
+    for attrname,attrtype in self.getFields():
       attr=getattr(self,attrname)
       # shorcut ignores
       if attrname in self.expectedValues:
@@ -439,18 +473,18 @@ class LoadableMembers(ctypes.Structure):
       log.debug("%s %s not loadable  bool(attr) = %s"%(attrname,attrtype, bool(attr)) )
       return True
     # load it, fields are valid
-    if isStructType(attr):
-      log.debug('%s %s is STRUCT'%(attrname,attrtype) )
+    if isStructType(attrtype):
+      log.debug('st: %s %s is STRUCT'%(attrname,attrtype) )
       if not attr.loadMembers(mappings, maxDepth+1):
-        log.debug("%s %s not valid, erreur while loading inner struct "%(attrname,attrtype) )
+        log.debug("st: %s %s not valid, erreur while loading inner struct "%(attrname,attrtype) )
         return False
-      log.debug("%s %s inner struct LOADED "%(attrname,attrtype) )
+      log.debug("st: %s %s inner struct LOADED "%(attrname,attrtype) )
       return True
     # maybe an array
-    if isBasicTypeArrayType(attr):
+    if isBasicTypeArray(attr):
       return True
-    if isArrayType(attr):
-      log.debug('%s is arraytype %s recurse load'%(attrname,repr(attr)) )#
+    if isArrayType(attrtype):
+      log.debug('a: %s is arraytype %s recurse load'%(attrname,repr(attr)) )#
       attrLen=len(attr)
       if attrLen == 0:
         return True
@@ -461,7 +495,7 @@ class LoadableMembers(ctypes.Structure):
       return True
     # we have PointerType here . Basic or complex
     # exception cases
-    if isCStringPointer(attr) : 
+    if isCStringPointer(attrtype) : 
       # can't use basic c_char_p because we can't load in foreign memory
       attr_obj_address = getaddress(attr.ptr)
       setattr(self,'__'+attrname,attr_obj_address)
@@ -480,7 +514,7 @@ class LoadableMembers(ctypes.Structure):
         log.warning('buffer size was too small for this CString')
       attr.string = txt
       return True
-    elif isPointerType(attr): # not functionType, it's not loadable
+    elif isPointerType(attrtype): # not functionType, it's not loadable
       _attrname='_'+attrname
       _attrType=self.classRef[attrtype]
       attr_obj_address=getaddress(attr)
@@ -523,33 +557,34 @@ class LoadableMembers(ctypes.Structure):
     The returned string should be python-compatible...
     '''
     s="%s # %s\n"%(prefix,repr(self) )
-    _fieldsTuple = [ (f[0],f[1]) for f in self._fields_] 
-    for field,typ in _fieldsTuple:
+    for field,typ in self.getFields():
       attr=getattr(self,field)
       s+=self._attrToString(attr,field,typ,prefix)
     return s
     
-  def _attrToString(self,attr,field,typ,prefix):
+  def _attrToString(self,attr,field,attrtype,prefix):
     s=''
-    if isStructType(attr):
+    if isStructType(attrtype):
       s=prefix+'"%s": {\t%s%s},\n'%(field, attr.toString(prefix+'\t'),prefix )  
-    elif isFunctionType(attr):
+    elif isFunctionType(attrtype):
       s=prefix+'"%s": 0x%lx, #(FIELD NOT LOADED)\n'%(field, getaddress(attr) )   # only print address in target space
-    elif isBasicTypeArrayType(attr): ## array of something else than int      
+    elif isBasicTypeArray(attr): ## array of something else than int      
       #log.warning(field)
       s=prefix+'"%s": b%s,\n'%(field, repr(array2bytes(attr)) )  
       #s=prefix+'"%s" :['%(field)+','.join(["0x%lx"%(val) for val in attr ])+'],\n'
-    elif isArrayType(attr): ## array of something else than int/byte
+    elif isArrayType(attrtype): ## array of something else than int/byte
       # go through each elements, we hardly can make a array out of that...
       s=prefix+'"%s" :{'%(field)
-      typ=type(attr[0])
+      eltyp=type(attr[0])
       for i in range(0,len(attr)):
-        s+=self._attrToString( attr[i], i, typ, '')
+        s+=self._attrToString( attr[i], i, eltyp, '')
       s+='},\n'
       #s=prefix+'"%s" :['%(field)+','.join(["%s"%(val) for val in attr ])+'],\n'
-    elif isPointerType(attr):
+    elif isPointerType(attrtype):
       if not bool(attr) :
         s=prefix+'"%s": 0x%lx,\n'%(field, getaddress(attr) )   # only print address/null
+      elif isVoidPointerType(attrtype) :
+        s=prefix+'"%s": 0x%lx, #Void pointer NOT LOADED\n'%(field, attr )   # only print address/null
       elif not is_address_local(attr) :
         s=prefix+'"%s": 0x%lx, #(FIELD NOT LOADED)\n'%(field, getaddress(attr) )   # only print address in target space
       else:
@@ -557,15 +592,15 @@ class LoadableMembers(ctypes.Structure):
         contents=attr.contents
         if type(self) == type(contents):
           s=prefix+'"%s": { #(0x%lx) -> %s\n%s},\n'%(field, getaddress(attr), type(attr.contents), prefix) # use struct printer
-        elif isStructType(contents): # do not enter in lists
+        elif isStructType(type(contents)): # do not enter in lists
           s=prefix+'"%s": { #(0x%lx) -> %s%s},\n'%(field, getaddress(attr), attr.contents.toString(prefix+'\t'),prefix) # use struct printer
-        elif isPointerType(contents):
+        elif isPointerType(type(contents)):
           s=prefix+'"%s": { #(0x%lx) -> %s%s},\n'%(field, getaddress(attr), self._attrToString(attr.contents, None, None, prefix+'\t'), prefix ) # use struct printer
         else:
           s=prefix+'"%s": { #(0x%lx) -> %s\n%s},\n'%(field, getaddress(attr), attr.contents, prefix) # use struct printer
-    elif isCStringPointer(attr):
+    elif isCStringPointer(attrtype):
       s=prefix+'"%s": "%s" , #(CString)\n'%(field, attr.string)  
-    elif isBasicType(attr): # basic, ctypes.* !Structure/pointer % CFunctionPointer?
+    elif isBasicType(attrtype): # basic, ctypes.* !Structure/pointer % CFunctionPointer?
       s=prefix+'"%s": %s, \n'%(field, repr(attr) )  
     else: # wtf ?
       s=prefix+'"%s": %s, # Unknown/bug DEFAULT repr\n'%(field, repr(attr) )  
@@ -573,38 +608,37 @@ class LoadableMembers(ctypes.Structure):
 
   def __str__(self):
     s=repr(self)+'\n'
-    _fieldsTuple = [ (f[0],f[1]) for f in self._fields_] 
-    for field,typ in _fieldsTuple:
+    for field,attrtype in self.getFields():
       attr=getattr(self,field)
-      if isStructType(attr):
+      if isStructType(attrtype):
         s+='%s (@0x%lx) : {\t%s}\n'%(field,ctypes.addressof(attr), attr )  
-      elif isFunctionType(attr):
+      elif isFunctionType(attrtype):
           s+='%s (@0x%lx) : 0x%lx (FIELD NOT LOADED)\n'%(field,ctypes.addressof(attr), getaddress(attr) )   # only print address in target space
-      elif isBasicTypeArrayType(attr):
+      elif isBasicTypeArray(attr):
         try:
           s+='%s (@0x%lx) : %s\n'%(field,ctypes.addressof(attr), repr(array2bytes(attr)) )  
         except IndexError,e:
           log.error( 'error while reading %s %s'%(repr(attr),type(attr)) )
           
-      elif isArrayType(attr): ## array of something else than int
+      elif isArrayType(attrtype): ## array of something else than int
         s+='%s (@0x%lx)  :['%(field, ctypes.addressof(attr),)+','.join(["%s"%(val) for val in attr ])+'],\n'
         continue
-      elif isPointerType(attr):
-        if not bool(attr) :
-          s+='%s (@0x%lx) : 0x%lx\n'%(field,ctypes.addressof(attr), getaddress(attr) )   # only print address/null
-        elif not is_address_local(attr) :
-          s+='%s (@0x%lx) : 0x%lx (FIELD NOT LOADED)\n'%(field,ctypes.addressof(attr), getaddress(attr) )   # only print address in target space
-        elif type(self) == type(attr.contents): # do not recurse in lists
-          s+='%s (@0x%lx) : (0x%lx) -> {%s}\n'%(field, ctypes.addressof(attr), getaddress(attr), repr(attr.contents) ) # use struct printer
-        else:
-          s+='%s (@0x%lx) : (0x%lx) -> {%s}\n'%(field, ctypes.addressof(attr), getaddress(attr), attr.contents) # use struct printer
-      elif isCStringPointer(attr):
+      elif isCStringPointer(attrtype):
         if not bool(attr) :
           s+='%s (@0x%lx) : 0x%lx\n'%(field,ctypes.addressof(attr), getaddress(attr) )   # only print address/null
         elif not is_address_local(attr) :
           s+='%s (@0x%lx) : 0x%lx (FIELD NOT LOADED)\n'%(field,ctypes.addressof(attr), getaddress(attr) )   # only print address in target space
         else:
           s+='%s (@0x%lx) : %s (CString) \n'%(field,ctypes.addressof(attr), attr.string)  
+      elif isPointerType(attrtype) and not isVoidPointerType(attrtype): # bug with CString
+        if not bool(attr) :
+          s+='%s (@0x%lx) : 0x%lx\n'%(field, ctypes.addressof(attr),   getaddress(attr) )   # only print address/null
+        elif not is_address_local(attr) :
+          s+='%s (@0x%lx) : 0x%lx (FIELD NOT LOADED)\n'%(field, ctypes.addressof(attr), getaddress(attr) )   # only print address in target space
+        elif type(self) == type(attr.contents): # do not recurse in lists
+          s+='%s (@0x%lx) : (0x%lx) -> {%s}\n'%(field, ctypes.addressof(attr), getaddress(attr), repr(attr.contents) ) # use struct printer
+        else:
+          s+='%s (@0x%lx) : (0x%lx) -> {%s}\n'%(field, ctypes.addressof(attr), getaddress(attr), attr.contents) # use struct printer
       elif type(attr) is long or type(attr) is int:
         s+='%s : %s\n'%(field, hex(attr) )  
       else:
@@ -618,15 +652,15 @@ class LoadableMembers(ctypes.Structure):
     reference should be handled nicely.
     '''
     # get self class.
-    #log.info("%s %s %s_py"%(self.__class__.__module__, sys.modules[self.__class__.__module__], self.__class__.__name__) )
+    #log.debug("%s %s %s_py"%(self.__class__.__module__, sys.modules[self.__class__.__module__], self.__class__.__name__) )
     my_class=getattr(sys.modules[self.__class__.__module__],"%s_py"%(self.__class__.__name__) )
     my_self=my_class()
     #keep ref
     if hasRef(my_class, ctypes.addressof(self) ):
       return getRef(my_class, ctypes.addressof(self) )
+    # we are saving us in a partially resolved state, to keep from loops.
     keepRef(my_self, my_class, ctypes.addressof(self) )
-    _fieldsTuple = [ (f[0],f[1]) for f in self._fields_] 
-    for field,typ in _fieldsTuple:
+    for field,typ in self.getFields():
       attr=getattr(self,field)
       member=self._attrToPyObject(attr,field,typ)
       setattr(my_self, field, member)
@@ -634,39 +668,39 @@ class LoadableMembers(ctypes.Structure):
     setattr(my_self, '_ctype_', type(self))
     return my_self
     
-  def _attrToPyObject(self,attr,field,typ):
-    if isStructType(attr):
+  def _attrToPyObject(self,attr,field,attrtype):
+    if isStructType(attrtype):
       obj=attr.toPyObject()
-    elif isBasicTypeArrayType(attr): ## array of basic types
+    elif isBasicTypeArray(attr): ## array of basic types
       obj=array2bytes(attr)
-    elif isArrayType(attr): ## array of something else than int/byte
+    elif isArrayType(attrtype): ## array of something else than int/byte
       obj=[]
-      typ=type(attr[0])
+      eltyp=type(attr[0])
       for i in range(0,len(attr)):
-        obj.append(self._attrToPyObject( attr[i], i, typ) )
-    elif isPointerType(attr):
+        obj.append(self._attrToPyObject( attr[i], i, eltyp) )
+    elif isPointerType(attrtype):
       if not bool(attr) :
         obj=(None,None)
       elif not is_address_local(attr) :
         obj=(None,getaddress(attr) )
       else:
         contents=attr.contents
-        if isStructType(contents) :
+        if isStructType(type(contents)) :
           attr_py_class = getattr(sys.modules[contents.__class__.__module__],"%s_py"%(contents.__class__.__name__) )
           cache = getRef(attr_py_class, getaddress(attr) )
           if cache:
             return cache
           obj=contents.toPyObject()
-        elif isPointerType(contents):
+        elif isPointerType(type(contents)):
           obj=self._attrToPyObject(contents,None,None)
         else: # pointer vers autre chose, le repr() est le seul choix.
           #obj=repr(contents)
           obj=contents
-    elif isCStringPointer(attr):
+    elif isCStringPointer(attrtype):
       obj=attr.string
-    elif isFunctionType(attr):
+    elif isFunctionType(attrtype):
       obj = repr(attr)
-    elif isBasicType(attr) and isCTypes(attr):
+    elif isBasicType(attrtype) and isCTypes(attr):
       obj = attr.value
     else:
       obj = attr
@@ -730,6 +764,7 @@ class pyObj(object):
       if self._attrFindCtypes(attr, attrname,typ ):
         log.warning('Found a ctypes in %s'%(attrname))
         ret = True
+    return ret
 
   def _attrFindCtypes(self, attr, attrname, typ):
     ret = False
@@ -749,7 +784,7 @@ class pyObj(object):
 
   def __iter__(self):
     ''' iterate on a instance's type's _fields_ members following the original type field order '''
-    for k,typ in [ (f[0],f[1]) for f in self._ctype_._fields_]:
+    for k,typ in self._ctype_.getFields():
       v = getattr(self,k)
       yield (k,v,typ)
     pass
@@ -757,7 +792,7 @@ class pyObj(object):
 def findCtypesInPyObj(obj):
   ''' check function to help in unpickling errors correction '''
   ret = False
-  if isinstance(obj, pyObj):
+  if hasattr(obj, 'findCtypes'):
     if obj.findCtypes():
       log.warning('Found a ctypes in array/tuple')
       return True
@@ -766,6 +801,8 @@ def findCtypesInPyObj(obj):
       if findCtypesInPyObj(el):
         log.warning('Found a ctypes in array/tuple')
         return True
+  elif isCTypes(obj):
+    return True
   return False
       
 import inspect,sys
@@ -837,7 +874,7 @@ def registerModule( targetmodule ):
     return
   _registered = 0
   for klass,typ in inspect.getmembers(targetmodule, inspect.isclass):
-    if typ.__module__.startswith(targetmodule.__name__):
+    if typ.__module__.startswith(targetmodule.__name__) and issubclass(typ, ctypes.Structure):
       register( typ )
       _registered += 1
   # create POPO's
